; #########################################################################

DecToQW    proc syscall

  ; ----------------------------------------------------------
  ; Convert positive non-zero decimal string into QWORD value
  ; by IanB

  ; On entry:
  ;     string pointer in ECX, string must be zero-terminated
  ; On exit:
  ;     converted value in EAX (low DWORD)/ EDX (high DWORD)
  ;     ECX is zero if there is an error in string or overflow,
  ;     else non-zero to show successful conversion
  ; EBX/EBP/EDI/ESI preserved
  ; ----------------------------------------------------------

        xor     eax, eax            ; use EAX as low-DWORD accumulator
        xor     edx, edx            ; use EDX as high-DWORD accumulator
        push    ebx                 ; use EBX as new byte input
        push    esi                 ; use these as convenience storage
        push    edi

        mov     al, BYTE PTR [ecx]
        add     ecx, 1
        xor     ebx, ebx
        sub     al, 30H
        jb      BadNumericString    ; not numeric digit
        cmp     al, 9
        ja      BadNumericString    ; not numeric digit

NewASCIIDigitLoop:
        mov     bl, BYTE PTR [ecx]
        add     ecx, 1
        xor     esi, esi
        xor     edi, edi

        test    bl, bl
        jz      EndASCIIDigitLoop   ; found string end
        sub     bl, 30H
        jb      BadNumericString    ; not numeric digit
        cmp     bl, 9
        ja      BadNumericString    ; not numeric digit
        cmp     edx, 19999999H
        ja      BadNumericString    ; value too big for QWORD when multiplied by 10
        je      @F                  ; test low DWORD if reached critical value

MultiplyQWBy10:
        add     eax, eax            ; current value * 2
        adc     edx, edx
        add     edi, eax            ; save this doubled value
        add     esi, edx
        add     eax, eax            ; current value * 4
        adc     edx, edx
        add     eax, eax            ; current value * 8
        adc     edx, edx
        add     eax, edi            ; current value * 10 = 8 + 2
        adc     edx, esi

        add     eax, ebx
        adc     edx, 0
        jc      BadNumericString    ; value too big for QWORD

        xor     ebx, ebx
        jmp     NewASCIIDigitLoop
@@:
        cmp     eax, 99999999H      ; did we reach the critical value?
        jbe     MultiplyQWBy10

BadNumericString:
        xor     eax, eax
        xor     edx, edx
        xor     ecx, ecx            ; show non-successful result by zero in ECX
        jmp     @F

EndASCIIDigitLoop:
        or      ecx, 0ffffffffH     ; show successful result by non-zero in ECX
@@:
        pop     edi
        pop     esi
        pop     ebx
        ret

DecToQW    endp

; #########################################################################
